
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, CHAT_FRAMES, CreateFrame, UIFrameFadeIn, UIFrameFadeOut

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat ScrollIndicator")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.scrollIndicator = {}

M.scrollOffset = {}

M.ScrollIndicatorMixin = {
	SetRevealed = function (self, reveal)
		self:SetEnabled(reveal)
		self:EnableMouse(reveal)
		UIFrameFadeIn(self, 0.25, self:GetAlpha(), reveal and 1 or 0)
	end,
}

function M:OnInitialize()
	for index = 1, #CHAT_FRAMES do
		self:CreateIndicator(_G["ChatFrame" .. index])
	end
end

function M:OnEnable()
	for index = 1, #CHAT_FRAMES do
		self:ConfigureChatFrame(_G["ChatFrame" .. index])
	end
end

function M:OnDisable()
	for index = 1, #CHAT_FRAMES do
		self:UpdateIndicator(_G["ChatFrame" .. index], 0)
	end

	table.wipe(self.scrollOffset)
end

-- Create a chat frame's scroll indicator button.
function M:CreateIndicator(ChatFrame)
	if not self.scrollIndicator[ChatFrame] then
		self.scrollIndicator[ChatFrame] = Mixin(CreateFrame("Button", nil, ChatFrame, "SixUI_BottomButtonTemplate"), self.ScrollIndicatorMixin)
	end
end

-- Update an indicator's visibility.
function M:UpdateIndicator(ChatFrame, scrollOffset)
	if self.scrollOffset[ChatFrame] ~= scrollOffset then
		self.scrollOffset[ChatFrame] = scrollOffset
		self.scrollIndicator[ChatFrame]:SetRevealed(scrollOffset ~= 0)
	end
end

-- Configure a chat frame to update the indicator.
function M:ConfigureChatFrame(ChatFrame)
	if not self:IsHooked(ChatFrame, "SetScrollOffset") then
		self:SecureHook(ChatFrame, "SetScrollOffset", "UpdateIndicator")
	end

	self:UpdateIndicator(ChatFrame, ChatFrame.scrollOffset)
end
